@@ -278,6 +278,7 @@ typedef		struct		_CELLTABLE {
 	¤ MORPHEME Ϥʤʤä */
 typedef         struct          _MRPH {
      U_CHAR             midasi[MIDASI_MAX];
+     U_CHAR             midasi2[MIDASI_MAX];
      U_CHAR             yomi[YOMI_MAX];
      U_CHAR             imis[IMI_MAX];
      CELL		*imi;
@@ -348,6 +349,7 @@ typedef struct _connect_cost {
     short p_no;     /* PROCESS_BUFFER Υǥå */
     short pos;
     int cost;     /*  */
+    int dakuon_flag;
 } CONNECT_COST;
 
 /*
