@@ -91,6 +91,7 @@
 #endif
 
 #include	<juman.h>
+#include	<const.h>
 
 /*
 ------------------------------------------------------------------------------
@@ -154,8 +155,12 @@ int             Show_Opt2;
 char		Show_Opt_tag[MIDASI_MAX];
 int		Show_Opt_jumanrc;
 int		Show_Opt_debug;
+int		Vocalize_Opt;
+int		Repetition_Opt;
+int		Normalized_Opt;
 
 U_CHAR	        String[LENMAX];
+U_CHAR	        NormalizedString[LENMAX];
 int             m_buffer_num;
 int             Jiritsu_buffer[CLASSIFY_NO + 1];
 int             undef_hinsi;
@@ -184,6 +189,27 @@ U_CHAR		midasi1[MIDASI_MAX]; /*  */
 U_CHAR		midasi2[MIDASI_MAX]; /*  */
 U_CHAR		yomi[MIDASI_MAX];    /* Ѥɤ */
 
+/* ƤǤ򸡺뤿Ѥ */
+/* ȾμưǧΤߤѤ */
+/* ܤʿ̾³ܤбҲ̾Ǥɬפ */
+U_CHAR          *dakuon[] = {"", "", "", "", "", "", "", "", "", "",
+			     "", "", "", "", "", "", "", "", "", "",
+			     "", "", "", "", "", "", "", "", "", "",
+			     "", "", "", "", "", "", "", "", "", "",
+			     "", "", "", "", "", "", "", "", "", "",
+			     "\0"};
+U_CHAR          *seion[]  = {"", "", "", "", "", "", "", "", "", "",
+			     "", "", "", "", "", "", "", "", "", "",
+			     "", "", "", "", "", "", "", "", "", "",
+			     "", "", "", "", "", "", "", "", "", "",
+			     "", "", "", "", "", "", "", "", "", "",
+			     "\0"};	     
+U_CHAR *lowercase[] = {"", "", "", "", "", "", "",
+		       "", "", "", "", "", "",
+		       "", "", "", "", "", "", "", "", "\0"};
+U_CHAR *uppercase[] = {"", "", "", "", "", "", "",
+		       "", "", "", "", "", "", "\0"};
+int normalized_lowercase = 7; /* lowercase[]ϰ(7""ޤǤо) */
 
 extern COST_OMOMI       cost_omomi;     /*k.n*/
 extern char             Jumangram_Dirname[FILENAME_MAX];  /*k.n*/
@@ -199,10 +225,10 @@ int	juman_close(void);
 void	realloc_mrph_buffer(void);
 void	realloc_process_buffer(void);
 void    read_class_cost(CELL *cell); /* k.n */
-static BOOL    katuyou_process(int position, int *k, MRPH mrph, int *length);
+static BOOL    katuyou_process(int position, int *k, MRPH mrph, int *length, int normal_flag);
 int     search_all(int position);
-int     take_data(int pos, char **pbuf);
-char *	_take_data(char *s, MRPH *mrph);
+int     take_data(int pos, char **pbuf, int dakuon_flag, int normal_flag);
+char *	_take_data(char *s, MRPH *mrph, int dakuon_flag, int normal_flag);
 int 	numeral_decode(char **str);
 int 	numeral_decode2(char **str);
 void 	hiragana_decode(char **str, char *yomi);
@@ -233,7 +259,7 @@ int	_print_homograph_path(FILE* output, int pbuf_start, int new_p);
 
 int	pos_match_process(int pos, int p_start);
 int	pos_right_process(int position);
-int	check_connect(int pos_start, int m_num);
+int	check_connect(int pos_start, int m_num, int dakuon_flag);
 int	juman_sent(void);
 
 /*
@@ -244,7 +270,6 @@ int	juman_sent(void);
 
 void changeDictionary(int number)
 {
-
     db = DicFile.dic[number];
     DicFile.now = number;
 }
@@ -487,15 +512,18 @@ void read_class_cost(CELL *cell)
 	PROCEDURE: <katuyou_process>       >>> changed by T.Nakamura <<<
 ------------------------------------------------------------------------------
 */
-static BOOL katuyou_process(int position, int *k, MRPH mrph, int *length)
+static BOOL katuyou_process(int position, int *k, MRPH mrph, int *length, int normal_flag)
 {
      while (Form[mrph.katuyou1][*k].name) {
 	  if (compare_top_str1(Form[mrph.katuyou1][*k].gobi,
-			      String + position + mrph.length)) {
+			      String + position + mrph.length) ||
+	      normal_flag && /* ɽ */
+	      compare_top_str1(Form[mrph.katuyou1][*k].gobi,
+			       NormalizedString + position + mrph.length)) {
 	       *length     = mrph.length + 
 		    strlen(Form[mrph.katuyou1][*k].gobi);
 	       return TRUE;
-	  } else {
+   	  } else {
 	       (*k)++;
 	  }
      }
@@ -510,35 +538,173 @@ static BOOL katuyou_process(int position, int *k, MRPH mrph, int *length)
 int search_all(int position)
 {
     MRPH        mrph;
-    int		len;
     int         dic_no;
     int         jmp ;
-    char 	length_info[4];
     int		i;
     char	*pbuf;
+    U_CHAR      buf[LENMAX];
 
     for(dic_no = 0 ; dic_no < DicFile.number ; dic_no++) {
 	changeDictionary(dic_no);
-	
-	/* ѥȥꥷڤǤ򸡺 */
+
 	pat_buffer[0] = '\0';
+
+	/* ѥȥꥷڤǤ򸡺 */
 	pat_search(db, String + position, &DicFile.tree_top[dic_no],
 		   pat_buffer);
 	pbuf = pat_buffer;
 
 	while (*pbuf != '\0') {
-	    if (take_data(position, &pbuf) == FALSE) return FALSE;
+	    if (take_data(position, &pbuf, 0, 0) == FALSE) return FALSE;
+	}
+
+	/* ѥȥꥷڤǤ򸡺(ɽ) by sasano */
+	if (Normalized_Opt) {
+	    if (strncmp(String + position, NormalizedString + position, NORMALIZED_LENGTH * 2)) {
+		pat_buffer[0] = '\0';
+		pat_search(db, NormalizedString + position, &DicFile.tree_top[dic_no],
+			   pat_buffer);
+		pbuf = pat_buffer;
+		
+		while (*pbuf != '\0') {
+		    if (take_data(position, &pbuf, 0, 1) == FALSE) return FALSE;
+		}
+	    }
+	}
+
+	if (Vocalize_Opt) {
+	    /* Ǥб뤿ν by sasano
+	       ȤꤢϤޤ줬ä
+	       ٤бϤޤ⸡Ƥ
+	       take_dataʲľ̾졢ޤưϢѷǤʤɤ¤
+	       ˥ڥʥƥ򤫤Ƥ("dakuon_flag"Ǹǽ)
+	    */
+	    for (i = 10; *dakuon[i]; i++) {
+		if (!strncmp(String + position, dakuon[i], 2)) {
+		    sprintf(buf, "%s%s", seion[i], String + position + 2);
+
+		    /* ѥȥꥷڤǤ򸡺 */	
+		    pat_buffer[0] = '\0';
+		    pat_search(db, buf, &DicFile.tree_top[dic_no], pat_buffer);
+		    pbuf = pat_buffer;
+
+		    while (*pbuf != '\0') {
+			if (take_data(position, &pbuf, i, 0) == FALSE) return FALSE;
+		    }
+		    break;
+		}
+	    }
 	}
     }
+    pat_buffer[0] = '\0';
+
+    /* Ҥ餬ʡʤη֤ɽθȤ */
+    if (Repetition_Opt) {
+	int code = check_code(String, position);
+	if (code == HIRAGANA || code == KATAKANA) {
+	    recognize_repetition(String + position, pat_buffer, code);
+	}
+    }   
+    pbuf = pat_buffer;
+    while (*pbuf != '\0') {
+	if (take_data(position, &pbuf, 0, 0) == FALSE) return FALSE;
+    }
+
     return TRUE;
 }
 
+/*
+------------------------------------------------------------------------------
+        PROCEDURE: <recognize_repetition>   >>> Added by Ryohei Sasano <<<
+------------------------------------------------------------------------------
+*/
+int recognize_repetition(char *key, char *rslt, int orig_code)
+{
+    int i, len, code, next_code, weight, con_tbl;
+    int key_length = strlen(key); /* ʸƤ */
+    U_CHAR *buf, midasi[LENMAX];
+
+    code = orig_code;
+    for (len = 2; len < 5; len++) {
+
+	if (key_length < len * 4) return 0;
+	
+	/* ʸϤޤΤԲ */
+	for (i = 0; *lowercase[i]; i++) {
+	    if (!strncmp(key, lowercase[i], 2)) return 0;
+	}
+
+	next_code = check_code(key, len * 2 - 2);
+
+	/* ʿ̾Ҳ̾""ʳޤΤԲ */
+	if (next_code != CHOON && next_code != HIRAGANA
+	    && next_code != KATAKANA) return 0;
+	if (next_code != CHOON) {
+	    if (next_code != code) return 0;
+	    code = next_code;
+	}
+
+	if (!strncmp(key, key + len * 2, len * 2)) {
+	    
+	    strncpy(midasi, key, len * 4);
+ 	    midasi[len * 4] = '\0';
+	    buf = midasi;	    
+
+	    /* Ūˤϸ㤷ƥڥʥƥͿ */
+	    weight = REPETITION_COST*len+0x20;
+
+	    /* ʸä1ʸʬޥʥ+ܡʥ */
+	    for (i = 2; *lowercase[i]; i++) {
+		if (strstr(buf, lowercase[i])) break;
+	    }
+	    if (*lowercase[i]) {
+		if (len == 2) return 0; /* 1η֤϶ػ */		
+		weight -= REPETITION_COST + LOWERCASE_BONUS;
+	    }
+
+	    /* äܡʥ */
+	    for (i = 0; *dakuon[i]; i++) {
+		if (strstr(buf, dakuon[i])) break;
+	    }
+	    if (*dakuon[i]) {
+		weight -= DAKUON_BONUS;
+		/* ƬäϤ˥ܡʥ */
+		if (!strncmp(buf, dakuon[i], 2)) weight -= DAKUON_BONUS;
+	    }
+
+	    /* ʤܡʥ */
+	    if (check_code(key, 0) == KATAKANA) weight -= KATAKANA_BONUS;
+    
+	    con_tbl = check_table_for_undef(8, 0); /* hinsi = 8, bunrui = 0 */
+	    sprintf(rslt, "%s\t%c%c%c%c%c%c%c", buf,
+		    8+0x20, /* hinsi = 8 */
+		    0+0x20, /* bunrui = 0 */
+		    0+0x20, /* katuyou1 = 0 */
+		    0+0x20, /* katuyou2 = 0 */
+		    weight, /* weight */
+		    con_tbl / (0x100-0x20)+0x20,
+		    con_tbl % (0x100-0x20)+0x20);
+	    rslt += len * 4 + 8;
+
+	    while (*buf) {
+		*(rslt++) = *buf;
+		*(rslt++) = *(buf+1);
+		buf += 2;
+	    }
+	    *(rslt++) = 0x20;
+	    *(rslt++) = 10+0x20;
+	    strcpy(rslt, "\"ưǧ\"\n");
+	    return len;
+	}
+    }
+}
+
 /*
 ------------------------------------------------------------------------------
         PROCEDURE: <take_data>                  >>> Changed by yamaji <<<
 ------------------------------------------------------------------------------
 */
-int take_data(int pos, char **pbuf)
+int take_data(int pos, char **pbuf, int dakuon_flag, int normal_flag)
 {
     unsigned char    *s;
     int     i, k, f, num;
@@ -552,8 +718,16 @@ int take_data(int pos, char **pbuf)
     s = *pbuf;
 
     k = 0 ;
+	    
     while ((mrph.midasi[k++] = *(s++)) != '\t') {}
     mrph.midasi[k-1] = '\0';
+    mrph.midasi2[0] = '\0';
+
+    if (dakuon_flag) {
+	strcpy(mrph.midasi2, mrph.midasi);
+	strncpy(mrph.midasi, String + pos, strlen(mrph.midasi));
+	mrph.midasi[strlen(mrph.midasi)] = '\0';
+    }
 
     if (*s == 0xff) { /* Ϣä */
 
@@ -565,7 +739,7 @@ int take_data(int pos, char **pbuf)
 	
 	pos_bak = pos;
 
-	s = _take_data(s , &mrph);
+	s = _take_data(s, &mrph, dakuon_flag, normal_flag);
 	rengo_con_tbl = mrph.con_tbl;
 	rengo_weight  = mrph.weight;
 	num = mrph.bunrui;
@@ -577,7 +751,7 @@ int take_data(int pos, char **pbuf)
 	    while ((new_mrph->midasi[k++] = *(s++)) != ' ') {}
 	    new_mrph->midasi[k-1] = '\0';
 
-	    s = _take_data(s , new_mrph);
+	    s = _take_data(s, new_mrph, dakuon_flag, normal_flag);
 
 	    length = strlen(new_mrph->midasi);
 	    if (Class[new_mrph->hinsi][new_mrph->bunrui].kt) /* Ѥ */
@@ -604,7 +778,7 @@ int take_data(int pos, char **pbuf)
 		/* ηǤѤ */
 		k2 = strlen(new_mrph->midasi) ? 1 : 2;
 		co = 0;
-		while (katuyou_process(pos, &k2, *new_mrph, &length)) {
+		while (katuyou_process(pos, &k2, *new_mrph, &length, 0)) {
 		    add_list[co++] = k2;
 		    k2++;
 		}
@@ -639,7 +813,7 @@ int take_data(int pos, char **pbuf)
 			printf("\n");
 		    }
 		    
-		    check_connect(pos, m_buffer_num);
+		    check_connect(pos, m_buffer_num, dakuon_flag);
 		    if (p_buffer_num == pnum_bak) break;
 
 		    p_buffer[pnum_bak].end = pos + new_mrph->length;
@@ -714,17 +888,24 @@ int take_data(int pos, char **pbuf)
 	}
 
     } else {           /* ̤ηǤä */
-	s = _take_data(s , &mrph);
+	s = _take_data(s, &mrph, dakuon_flag, normal_flag);
 
 	if ( Class[mrph.hinsi][mrph.bunrui].kt ) { /* Ѥ */
 	    if ( mrph.katuyou2 == 0 ) {   /* 촴 */
 		k2 = 1;
-		while (katuyou_process(pos, &k2, mrph, &length)) {
+		while (katuyou_process(pos, &k2, mrph, &length, normal_flag)) {
+		    /* Ρɤ2ʾ塢ġlengthоݤξʸޤΤߺ */
+		    if (normal_flag &&
+			(length == 2 ||
+			 !strncmp(String + pos, NormalizedString + pos, length))) {
+			k2++;
+			continue;
+		    }
 		    m_buffer[m_buffer_num] = mrph;
 		    m_buffer[m_buffer_num].katuyou2 = k2;
 		    m_buffer[m_buffer_num].length = length;
 		    m_buffer[m_buffer_num].con_tbl += (k2 - 1);
-		    check_connect(pos, m_buffer_num);
+		    check_connect(pos, m_buffer_num, dakuon_flag);
 		    if (++m_buffer_num == mrph_buffer_max)
 			realloc_mrph_buffer();
 		    k2++;
@@ -732,28 +913,34 @@ int take_data(int pos, char **pbuf)
 	    } else {                         /* 촴ʤ */
 		m_buffer[m_buffer_num] = mrph;
 		m_buffer[m_buffer_num].midasi[0] = '\0';
+		m_buffer[m_buffer_num].midasi2[0] = '\0';
 		m_buffer[m_buffer_num].yomi[0]  = '\0';
-		check_connect(pos, m_buffer_num);
+		check_connect(pos, m_buffer_num, dakuon_flag);
 		if (++m_buffer_num == mrph_buffer_max)
 		    realloc_mrph_buffer();
 	    }
 	} else {	                                 /* Ѥʤ */
-	    m_buffer[m_buffer_num] = mrph;
-	    check_connect(pos, m_buffer_num);
-	    new_mrph_num = m_buffer_num;
-	    if (++m_buffer_num == mrph_buffer_max)
-		realloc_mrph_buffer();
+	    if (!normal_flag ||  
+		/* Ρɤ2ʾ塢ġlengthоݤξʸޤΤߺ */
+		strlen(mrph.midasi) > 2 && 
+		strncmp(String + pos, NormalizedString + pos, strlen(mrph.midasi))) {
+		
+		m_buffer[m_buffer_num] = mrph;
+		check_connect(pos, m_buffer_num, dakuon_flag);
+		new_mrph_num = m_buffer_num;
+		if (++m_buffer_num == mrph_buffer_max)
+		    realloc_mrph_buffer();	    
 
 #ifdef NUMERIC_P
-	    if (suusi_word(pos, new_mrph_num) == FALSE)
-		return FALSE;
+		if (suusi_word(pos, new_mrph_num) == FALSE)
+		    return FALSE;
 #endif
 #ifdef THROUGH_P
-	    if (through_word(pos, new_mrph_num) == FALSE)
-		return FALSE;
+		if (through_word(pos, new_mrph_num) == FALSE)
+		    return FALSE;
 #endif
+	    }
 	}
-
     }
     *pbuf = s;
     return TRUE;
@@ -765,10 +952,10 @@ int take_data(int pos, char **pbuf)
 ------------------------------------------------------------------------------
 */
 
-char *_take_data(char *s , MRPH *mrph)
+char *_take_data(char *s, MRPH *mrph, int dakuon_flag, int normal_flag)
 {
     int		i, j, k = 0;
-    char	c;
+    char	c, *rep;
     
     mrph->hinsi    = numeral_decode(&s);
     mrph->bunrui   = numeral_decode(&s);
@@ -778,21 +965,94 @@ char *_take_data(char *s , MRPH *mrph)
     mrph->con_tbl  = numeral_decode2(&s);
     hiragana_decode(&s, mrph->yomi);
     mrph->length   = strlen(mrph->midasi);
-
+    
     if (*s != ' ' && *s != '\n') { /* ̣󤢤 */
 	j = numeral_decode(&s);
 	for (i = 0; i < j; i++) mrph->imis[k++] = *(s++);
 	mrph->imis[k] = '\0';
     }
     s++;
-    if (k == 0) strcpy(mrph->imis, NILSYMBOL);
+    
+    if (dakuon_flag) {
+	/* 饤ޥˡ§ */
+	/* ȤޤǤʤ */
+	/* 㳰Ȥʤ"Ϥ"ˤϡĤȤfeatureͿн褷Ƥ */
+	for (i = 10; *dakuon[i]; i++) {
+	    if (strstr(mrph->midasi + 2, dakuon[i])) break;
+	}
+	/* 饤ޥˡ§˳촴Τʤ졢Ǥ1ʸä */
+	/* νϤʤ(=礭ʥڥʥƥͿ) */
+	if (!strstr(mrph->imis, "") && *dakuon[i] || 
+	    mrph->katuyou2 || 
+ 	    (mrph->length == 2) && !Class[mrph->hinsi][mrph->bunrui].kt) {
+	    mrph->weight = 255;
+	}
+	/* Ϣ¸Τ */
+	/* ΤɽɽκǽʸǤʤΤԲ */
+	else if ((rep = strstr(mrph->imis, "ɽɽ:")) &&
+		 check_code(rep, 9) == KATAKANA) {
+	    mrph->weight = 255;
+	}
+	else {
+	    /* ư */
+	    if (mrph->hinsi == 2) {
+		if (!strncmp(mrph->midasi, "", 2)) {
+		    mrph->weight += VERB_GA_VOICED_COST;
+		}
+		else {
+		    mrph->weight += VERB_VOICED_COST;
+		}
+	    }
+	    /* ̾ */
+	    else if (mrph->hinsi == 6 && (mrph->bunrui < 3 || mrph->bunrui > 7)) {
+		if (!strncmp(mrph->midasi, "", 2)) {
+		    mrph->weight += NOUN_GA_VOICED_COST;
+		}
+		else {
+		    mrph->weight += NOUN_VOICED_COST;
+		}
+	    }
+	    /* ƻ */
+	    else if (mrph->hinsi == 3) {
+		mrph->weight += ADJECTIVE_VOICED_COST;
+	    }
+	    /* ¾ */
+	    /* ¾ʻǤĤȤ̣ǤвϤǤ褦ˤ뤿 */
+	    /* 줪"餤"ꤷƤ뤬ȤꤢϻѤ */
+	    else if (strstr(mrph->imis, "")) {
+		mrph->weight += OTHER_VOICED_COST;
+	    }
+	    else {
+		mrph->weight = 255;
+	    }
+	}
+	
+	/* ɤߤҲ̾ξʿ̾ˤ */
+	strncpy(mrph->yomi, dakuon[(dakuon_flag/2)*2], 2);
+	
+	if (k == 0) {
+	    strcpy(mrph->imis, "\"\"");
+	}
+	else {
+	    mrph->imis[k - 1] = '\0';
+	    strcat(mrph->imis, " \"");
+	}
+	k++;
+    }
 
+    /* Τ˥ڥʥƥ(ʸ줿ɽ) */
+    if (normal_flag) {
+	mrph->weight += NORMALIZED_COST;
+    }
+    
+    if (k == 0) strcpy(mrph->imis, NILSYMBOL);
+    
     return(s);
 }
 
 int numeral_decode(char **str)
 {
-    unsigned char *s;
+  unsigned char *s;
 
     s = *str;
     if (*s < 0xf0) {
@@ -874,6 +1134,9 @@ int undef_word(int pos)
     } else {
 	end = pos;
 	while(1) {
+	    /* MIDASI_MAX̤ۤϺʤ */
+	    if (end - pos >= MIDASI_MAX - ((code == HANKAKU) ? 1 : 2)) break;
+
 	    end += (code == HANKAKU) ? 1 : 2;
 	    next_code = check_code(String, end);
 	    if (next_code == code ||
@@ -923,9 +1186,10 @@ int undef_word(int pos)
 	m_buffer[m_buffer_num].yomi[end - pos] = '\0';
     }
     m_buffer[m_buffer_num].weight = MRPH_DEFAULT_WEIGHT;
+    strcpy(m_buffer[m_buffer_num].midasi2, m_buffer[m_buffer_num].midasi);
     strcpy(m_buffer[m_buffer_num].imis, NILSYMBOL);
 
-    check_connect(pos, m_buffer_num);
+    check_connect(pos, m_buffer_num, 0);
     if (++m_buffer_num == mrph_buffer_max)
 	realloc_mrph_buffer();
 
@@ -1014,8 +1278,10 @@ int 	suusi_word(int pos , int m_num)
 
 	    if (strlen(pre_mrph->midasi)+strlen(new_mrph->midasi) >= MIDASI_MAX
 		|| strlen(pre_mrph->yomi)+strlen(new_mrph->yomi) >= YOMI_MAX) {
-		fprintf(stderr, "Too long suusi<%s>\n", String);
-		return FALSE;
+		/* MIDASI_MAXYOMI_MAXۤʬ䤹褦ѹ 08/01/15 */
+		/* fprintf(stderr, "Too long suusi<%s>\n", String);
+		   return FALSE; */
+		return TRUE;
 	    }
 	    m_buffer[m_buffer_num] = *pre_mrph;
 	    strcat(m_buffer[m_buffer_num].midasi , new_mrph->midasi);
@@ -1128,11 +1394,12 @@ int 	through_word(int pos , int m_num)
     
 int 	is_through(MRPH *mrph_p)
 {
-    if ((mrph_p->hinsi == kakko_hinsi &&
-	 (mrph_p->bunrui == kakko_bunrui1 || 
-	  mrph_p->bunrui == kakko_bunrui2))||
-	(mrph_p->hinsi == kuuhaku_hinsi && 
-	 mrph_p->bunrui == kuuhaku_bunrui))
+    if (
+	/* ̻ϤƩ򤷤ʤ 2007/12/06
+	(mrph_p->hinsi == kakko_hinsi && mrph_p->bunrui == kakko_bunrui1) ||
+	*/
+	(mrph_p->hinsi == kakko_hinsi && mrph_p->bunrui == kakko_bunrui2) ||
+	(mrph_p->hinsi == kuuhaku_hinsi && mrph_p->bunrui == kuuhaku_bunrui))
 	return TRUE;
     else 
 	return FALSE;
@@ -1180,7 +1447,7 @@ MRPH *prepare_path_mrph(int path_num , int para_flag)
     else
 	kigou[0] = '\0';
     strcpy(midasi1, mrph_p->midasi);
-    strcpy(midasi2, mrph_p->midasi);
+    strcpy(midasi2, *mrph_p->midasi2 ? mrph_p->midasi2 : mrph_p->midasi);
     strcpy(yomi, mrph_p->yomi);
     if ( (mrph_p->katuyou1 > 0) && (mrph_p->katuyou2 > 0) ) {
 	strcat(midasi1, Form[mrph_p->katuyou1][mrph_p->katuyou2].gobi);
@@ -1188,6 +1455,20 @@ MRPH *prepare_path_mrph(int path_num , int para_flag)
 	strcat(midasi2, Form[mrph_p->katuyou1][j].gobi);
 	strcat(yomi, Form[mrph_p->katuyou1][mrph_p->katuyou2].gobi_yomi);
     }
+
+    /* ʸˤɸɽ */
+    if (mrph_p->hinsi != 1 && mrph_p->hinsi != 15 &&
+	strncmp(String + p_buffer[path_num].start, midasi1, mrph_p->length)) {
+	strncpy(midasi1, String + p_buffer[path_num].start, mrph_p->length);
+	if (strcmp(mrph_p->imis, NILSYMBOL)) {
+	    mrph_p->imis[strlen(mrph_p->imis) - 1] = '\0';
+	    strcat(mrph_p->imis, " ʸ\"");
+	}
+	else {
+	    strcpy(mrph_p->imis, "\"ʸ\"");
+	}
+    }
+
     return mrph_p;
 }
 
@@ -1313,6 +1594,14 @@ void print_path_mrph(FILE* output, int path_num , int para_flag)
 	break;
 
     case Op_EE:
+	/* ƥΤĤʤɽ */
+	fprintf(output, "%d ", path_num);
+	for (i = 0; proc_p->path[i] != -1; i++) {
+	    if (i) fprintf(output, ";");
+	    fprintf(output, "%d", proc_p->path[i]);
+	}
+	fprintf(output, " ");
+
 	fprintf(output, "%d ", pos);
 	if (!strcmp(midasi1, "\\ ")) pos++; else pos += strlen(midasi1);
 	fprintf(output, "%d ", pos);
@@ -1634,7 +1923,7 @@ int pos_right_process(int pos)
 ------------------------------------------------------------------------------
 */
 
-int check_connect(int pos, int m_num)
+int check_connect(int pos, int m_num, int dakuon_flag)
 {
     static CHK_CONNECT_WK chk_connect[MAX_PATHES_WK];
     int chk_con_num;
@@ -1656,7 +1945,8 @@ int check_connect(int pos, int m_num)
     /* Ϣܥå˥ҥåȤСõԤʤ */
     c_cache = &connect_cache[rensetu_tbl[new_mrph->con_tbl].j_pos];
     if (Show_Opt_debug == 0) {
-	if (c_cache->pos == pos && c_cache->p_no > 0) {
+	if (c_cache->pos == pos && c_cache->dakuon_flag == dakuon_flag 
+	    && c_cache->p_no > 0) {
 	    p_buffer[p_buffer_num].score = c_cache->cost + class_score;
 	    p_buffer[p_buffer_num].mrph_p = m_num;
 	    p_buffer[p_buffer_num].start = pos;
@@ -1679,6 +1969,21 @@ int check_connect(int pos, int m_num)
 
 	c_score = check_matrix(left_con , right_con);
 
+	/* ΤľηǤ̾졢ޤưϢѷ̾ξΤ */
+	/* ľηǤξʿ̾1ʸȤʤΤԲ */
+	/* weight=255ΤȤԲ */
+	if (dakuon_flag &&
+	    (!(m_buffer[p_buffer[j].mrph_p].hinsi == 6 ||
+	       m_buffer[p_buffer[j].mrph_p].hinsi == 2 &&
+	       m_buffer[p_buffer[j].mrph_p].katuyou2 == 8 ||
+	       m_buffer[p_buffer[j].mrph_p].hinsi == 14 &&
+	       m_buffer[p_buffer[j].mrph_p].bunrui < 5) ||
+	     (m_buffer[p_buffer[j].mrph_p].hinsi != 14 &&
+	      check_code(m_buffer[p_buffer[j].mrph_p].midasi, 0) == HIRAGANA &&
+	      m_buffer[p_buffer[j].mrph_p].length == 2) ||
+	     new_mrph->weight == 255))
+	    c_score = 0;
+
 	if (c_score) {
 	    chk_connect[chk_con_num].pre_p = j;
 
@@ -1756,6 +2061,7 @@ int check_connect(int pos, int m_num)
     c_cache->p_no = p_buffer_num;
     c_cache->cost = best_score;
     c_cache->pos = pos;
+    c_cache->dakuon_flag = dakuon_flag;
 
     /* 괺٥ȥѥ1Ĥ0֤Ͽ */
     p_buffer[p_buffer_num].path[0] = chk_connect[best_score_num].pre_p;
@@ -1823,6 +2129,22 @@ int juman_sent(void)
     m_buffer_num = 1;
     p_buffer_num = 1;
     
+    /* ɽؤб */
+    strcpy(NormalizedString, String);
+    for (pos = 0; pos < length; pos+=next_pos) {
+	if (String[pos]&0x80) { /* Ѥξ */
+	    for (i = 0; i < normalized_lowercase; i++) {
+		if (!strncmp(String + pos, lowercase[i], 2)) {
+		    NormalizedString[pos] = uppercase[i][0];
+		    NormalizedString[pos+1] = uppercase[i][1];
+		}
+	    }
+	    next_pos = 2;
+	} else {
+	    next_pos = 1;
+	}
+    }
+
     for (pos = 0; pos < length; pos+=next_pos) {
 
 	p_start = pos_match_process(pos, p_start);
@@ -1852,7 +2174,7 @@ int juman_sent(void)
 	realloc_mrph_buffer();
 
     pos_match_process(pos, p_start);
-    if (check_connect(length, m_buffer_num-1) == FALSE)
+    if (check_connect(length, m_buffer_num-1, 0) == FALSE)
     return FALSE;
 
     return TRUE;
